#!/usr/bin/env python

"""
A simple example of parallel computation using message queues and the create
function.
"""

import pprocess
import time

# Array size and a limit on the number of processes.

N = 10
limit = 10
delay = 1

# Main program.

if __name__ == "__main__":

    # Initialise the communications queue with a limit on the number of
    # channels/processes.

    queue = pprocess.Queue(limit=limit)
    t = time.time()

    # Initialise an array.

    D = [0] * N * N

    # The parallel computation.
    # NOTE: Could use the with statement in the loop to package the
    # NOTE: try...finally functionality.

    print "Calculating..."
    for i in range(0, N):
        for j in range(0, N):
            ch = queue.create()
            if ch:
                try: # Calculation work.

                    time.sleep(delay)
                    ch.send((i, j, i * N + j))

                finally: # Important finalisation.

                    pprocess.exit(ch)

    print "Finishing..."
    for i, j, result in queue:
        D[i*N+j] = result

    # Show the result.

    print "Time taken:", time.time() - t
    for i in range(0, N):
        for result in D[i*N:i*N+N]:
            print result,
        print

# vim: tabstop=4 expandtab shiftwidth=4
